#include <omp.h>
#include <stdio.h>
#include <imsls.h>
#include <imsl.h>
#include <omp.h>
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <time.h>

using namespace std;

const int SA1 =10;					// numbers of years that an establishment lives
const int SP1 =9;					// number of skill level it can grow

// Main matrices

double   K[SP1][SA1];				//level of uboundital, constrained
double   P[SP1][SA1];				//prob of detecting cheating
double   X[SP1][SA1];				//payments to investor
double   T[SP1];					//productivity vector
double   pr[SP1][SA1];				//prob of that state.

// Number of constraints, choice vars and periods.

#define N1  SA1						// number of periods
#define T1  33	         			// number of constraints, LL and IC
#define E1  1						// number of equality constraints, zero profits
#define C1  68						// number of choice variables, 

// Useful functions

void   fcn(int n, double x[], int iact, double *result, int *ierr); // function that contains maximand and constraints
double Cf(double kx, double px);	// cost of monitoring 
double Lfm(double kx, double px);	// labor demand in monitoring
double Lfp(double kx, double theta);// labor demand in prod
double maxi(double x, double y);	// max btw two doubles

// Main parameters

int    period	= 5;					// meaning that 1 period is 5 years
double alpha	= 0.85;					// scale parameter in the prod function.
double omega	= (1.0/3.0);			// share of k in the prod function.
double gamma	= 2;					// exponent on (k/z) in cost of p fn.
double beta		= pow(0.98,period);		//manager's and investor's discount factor
double lbound	= 0;					// min on k
double delta	= 1- pow(1-0.07,period);// depreciation rate


double lambda = 0;


// United States
double A        = 2102.808;
double w		= 47500.827;					// economy wage // Mexico wage is 0.25, US is 1
double r		= 0.611;					// cost of capital // Same cost for Mexico 
double w_tilde  = 1;				    // cost for the US // Mexico cost is 0.6768, US is 1.1586

double z		;		// monitoring technology parameter. 
double ubound=12	;					// ubound on k


double th_c  = -1.01712;				// constant in theta profile
double th_a	 =  1.10411;				// slope in theta profile
double th_a2 = -0.27;				// exponent in theta profile
double th_a3 = 0.023;				// exponent in theta profile

double rho_intercept= .45;
double rho_slope    = -.035;
double rho_exponent = 1;

double sv_c  = .14025;
double sv_a  = -0.022;
double sv_a2 = 0.00115;

// Useful vectors

double sv[SP1-1];						// survival rate
double rho[SP1-1];						// prob of stall, from period to period
double stall[SP1-1];					// prob of being stall
double nostall[SP1-1];					// prob of not being stall
double Q[SP1-1];						// fixed cost

// Auxiliar

double  MM = 1.0e12;						// large #
double  AD = 1000000;						// large #
#define NDATA 11						// # of points in size distribution
#define N_OBSERVATIONS 55	// # of possible states

// File were I print the output

ofstream qstat  ("statistics.dat"); // file used to print data
double VVV;

int main()
{
	// Print screen the # of choice variables 
	cout << "# choice variables  = " << C1 << endl;

	// Fixed costs

	Q[0] = 0.0; Q[1] = 0; Q[2] = 0; Q[3] = 0; 
	Q[4] = 0;    Q[5] = 0; Q[6] = 0; Q[7] = 0;
	
	// Auxiliars to the maximization

	int ibtype = 0;					// initial guess on type
	double *x;						// vector that stores results
	static double xlb[C1], xub[C1]; // lower and upper bounds in choice variables
	static double xlbstar[C1], xubstar[C1]; // lower and upper bounds in choice variables
	static double xlbG[C1], xubG[C1]; // lower and upper bounds in choice variables
	
	double xguess[C1];				// guess on variable, uneducated
	double xguessini[C1];				// guess on variable, uneducated
	
	double xgstar[C1];				// guess on variable, educated
	double xgS[C1];					// guess on variable, very educated

	double xscale[C1];				// scale on variable, uneducated
	double xscalestar[C1];				// scale on variable, uneducated
//	double xscaleG[C1];				// scale on variable, uneducated
	double Vold = MM;				// store previous value of V
	double VVMAX = -MM;				// keep the max of V so far
	int	   nn = 0;					// number of large loops
	
	// Importing guess from file

	ifstream in ("guess.dat");
	for(int i=0;i<C1;i++)
	{		in >> xguess[i] ;
			//cout << xguess[i] << endl;
	}
	
	// Options for the maximization

	imsl_error_options( IMSL_SET_PRINT, IMSL_WARNING, 0, 
					    IMSL_SET_PRINT, IMSL_FATAL, 0, 
						IMSL_SET_PRINT,IMSL_TERMINAL, 0, 
						IMSL_SET_STOP, IMSL_FATAL, 0,
						IMSL_SET_STOP, IMSL_TERMINAL, 0,
						IMSL_SET_ERROR_FILE, stdout,0);

	// Print to screen main options
	
	cout << endl;
	cout    <<"beta = " << beta << endl;
	cout    <<"W = " << w << endl;
	cout   << "R = " << r << endl;
	cout   << "W~= " << w_tilde << endl;
	cout   <<"Min = " << lbound << endl;
	cout   <<"Cap = " << ubound << endl;
	cout << endl;
	
	int search_count = 1;
		
	qstat   << "W_tilde = " << w_tilde << endl;
	qstat    <<"Cap = " << ubound << endl;
					

	th_c = (-0.232046);				// constant in theta profile
	for(int in=0;in<1;in++)
	{
	th_a  = (0.14609);				// slope in theta profile
	for(int il=0;il<1;il++)
	{
	th_a2 = (-0.0424995);				// exponent in theta profile
	for(int ie=0;ie<1;ie++)
	{
	th_a3  = (0.0075);				// exponent in theta profile
	for(int ie2=0;ie2<1;ie2++)
	{
	rho_intercept = (0.31);				
	for(int ir=0;ir<1;ir++)
	{
	rho_slope= 1.0*(0);				
	for(int ir2=0;ir2<1;ir2++)
	{

	Q[0] = 0.255;
	z = 1.75; 
	lambda = 0.06;				
	ubound	= 12;


	cout    << "============================================================"<<endl;

	qstat   << "lambda = " << lambda << endl;
	cout   << "lambda = " << lambda << endl;	
	qstat   << "Z = " << z << endl;
	cout   << "Z = " << z << endl;
	qstat   << "Q = " << Q[0] << endl;
    cout   <<"Q	= " << Q[0] << endl;
	

	sv[0] = 1;
	for(int i=1;i<SA1;i++)
		sv[i] = sv[i-1]*pow(1 - (sv_c + sv_a*i + sv_a2*i*i), period);

	
	for(int i=0;i<SP1;i++)
		rho[i] = rho_intercept + rho_slope*pow(i, rho_exponent);

	// Print to screen profiles of rho and theta
	for(int i=0;i<SP1;i++)
	{	cout   << rho[i] <<" ";
		qstat << i<< ".rho = " << rho[i] << endl;
	}
	cout << endl;

	
	stall[0]   = rho[0];
	nostall[0] = 1.0 - rho[0];

	//cout << "y?"<<nostall[0] << endl;
	for(int i=1;i<SP1-1;i++)
	{	  stall[i] = nostall[i-1]*rho[i];
		  nostall[i] = nostall[i-1]*(1-rho[i]);
		  //cout << "nostall"<<i-1 <<" = " << nostall[i-1] << endl;
	}
	
	

	// Creating unconditional probability for each state

	double suma = 0;
	for(int i=0;i<SP1;i++)
		for(int j=0;j<SA1;j++)
			pr[i][j] = 0;

	for(int i=0;i<SP1;i++)
	{	for(int j=max(i-1,0);j<SA1;j++)
		{	
			pr[i][j] = sv[j];
			 
			if(i<=j)
			{	
				pr[i][j] = stall[i]*pr[i][j];
			}
			
			if(i==j+1)
			{	//cout << j <<" " << nostall[j] << endl;
				pr[i][j] = nostall[j]*pr[i][j];
			}
			
			if(i>j+1)
				pr[i][j] = 0;
			
			   suma = suma + pr[i][j] ;
			  //cout   << "pr("<< i<<","<<j<< ") = " << pr[i][j] << endl;
			  //double nada;
			  //cin >> nada;
		
		}
	}

	for(int j=SP1-1;j<SA1;j++)
	{	
		pr[SP1-1][j] = nostall[SP1-2]*sv[j];
		suma = suma + pr[SP1-1][j];
		//cout   << "pr("<< SP1-1<<","<<j<< ") = " << pr[SP1-1][j] << endl;
	}

	double measure_estab = suma;

	
	// Printing to screen the total mass of firms

//	cout << "Mass = " <<suma << endl;
	
	// Printing to screen the probability of each state

/*	for(int i=0;i<SP1;i++)
	{	for(int j=0;j<SA1;j++)
			cout   << pr[i][j] << " ";
		cout << endl;
	}*/
	// Printing to screen and file the survival probs

	for(int i=0;i<SA1;i++)
	{	//cout   << i<< ".sv = " << sv[i] << endl;
		qstat << i<< ".sv = " << sv[i] << endl;
	}



	for(int i=0;i<SP1;i++)
	{	
		T[i] = exp( th_c + th_a*(i) + th_a2*(i)*(i)+th_a3*(i)*(i)*(i) );
		cout  << " " <<  T[i]  ;
		qstat << "theta" << i<<" = " <<  T[i]  << endl;
	}
	cout << endl;

	nn= 0;

	VVMAX = -MM;
			
	while(nn<1) 
	{	
		
		ifstream in ("guess.dat");
		if(nn==0)
			for(int i=0;i<C1;i++)
				in >> xguess[i] ;
		
		int count = 0;
			
		// bounds and guess on probabilities
		for(int i=0;i<(SP1-1)*SA1-(1+2+3+4+5+6+7);i++)
		{	xlb[i] = 0;
			xub[i] = .99999999999999;
			xscale[i] = 10;
			count++;
		}
	// bounds and guess on uboundital
		for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7);i<(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1;i++)
		{	xlb[i] =  0;
			xub[i] = ubound*2;	
			if(nn>0)
				xguess[i] = .3*pow(i-((SP1-1)*SA1-(1+2+3+4+5+6+7))+1,3.0);
			xscale[i]=10;
			count++;
		}
			
		cout << endl;

		// bounds and guess on payments
		for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1; i< (SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1 + SP1-1;i++)
		{	xlb[i] = -10.0e7;
			xub[i] =  10.0e7; 
			if(nn>0)
				xguess[i] = .15*pow(i-((SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1)+1,3.0);
			xscale[i]=10;
			count++;
		}

		//cout << "# of choice variables initiated = " << count << endl;
			
			
		// setting the guess at the best so far
		if(nn>1)
			for(int i=0; i< C1;i++)
				xguess[i] = xguess[i]/(1+.2);

		if(nn>0)
			for(int i=0; i< C1;i++)
				xguess[i] = xguess[i]*.5 + xgS[i]*.5;
		

		nn++;
		
		double VVmax = -MM;
		int MAXI = 15;
		cout << "V = " ;

		for(int mm=1;mm<MAXI;mm++)
		{
			/* RUNNING THE OPTIMIZATION ROUTINE */	
			//imsl_omp_options(IMSL_SET_FUNCTIONS_THREAD_SAFE, 1, 0);
			x = imsl_d_constrained_nlp(fcn, T1 , 1, C1, ibtype, xlb, xub, IMSL_XGUESS, xguess,IMSL_ITMAX, 50000, 
			/*IMSL_TAU0, 10.e-5,
			IMSL_DEL0, 10.e-8,
			IMSL_SMALLW, 10.e-8,
			IMSL_DELMIN, 10.e-8,
			IMSL_SCFMAX, 10.e-16,*/
			0);
		
			
			for(int j=0;j<SP1;j++)
				for(int i=0;i<SA1;i++)
					P[j][i] = 0;
				
			int count = -1;
			for(int j=0;j<SP1-1;j++)
				for(int i=j;i<SA1;i++)
				{	count++;
					P[j][i] = x[count];
					//cout << "P"<<j<<i<<" = " << P[j][i] << endl;
				}
				
			for(int i=0;i<SP1-1;i++)
			{	count++;
				K[i+1][i] = x[count];
				K[i][i] = x[count];
				//cout << "K"<<i+1<<i<<" = " << K[i+1][i] << endl;
				//cout << "K"<<i<<i<<" = " << K[i][i] << endl;
			}

			//cout << " IRREVERSABILITY " << endl;
			for(int j=0;j<SP1-1;j++)
				for(int i=j+1;i<SA1;i++)
				{	K[j][i] = K[j][j];
					//cout << "K"<<j<<i<<" = " << K[j][i] << endl;
				}

			for(int i=SP1-1;i<SA1;i++)
			{	K[SP1-1][i] = K[SP1-1][SP1-2];
				//cout << "K"<<SP1-1<<i<<" = " << K[SP1-1][i] << endl;
			}

			for(int i=0;i<SP1-1;i++)
			{	count++;
				X[i+1][i] = x[count];
				//cout << "X"<<i+1<<i<<" = " << X[i+1][i] << endl;
			}

			double total;
			for(int j=0;j<SP1;j++)
			{	total = 0;
								for(int i=j;i<SA1;i++)
				{		X[j][i] = T[j]*pow(K[j][i],alpha);
						total = pr[j][i]*pow(beta,i-j)*T[j]*pow(K[j][i],alpha)*lambda + total;
						//X[j][i] = T[j]*pow(K[j][i],alpha)*(1.0-lambda);
						//cout << "X"<<j<<i<<" = " << X[j][i] << endl;
				}
				X[j][SA1-1] = T[j]*pow(K[j][SA1-1],alpha)-total/(pr[j][SA1-1]*pow(beta,SA1-1));
			}

			double V = 0;
			for(int j=0;j<SP1;j++)
				for(int i=max(j-1,0);i<SA1;i++)
				{	V = V + pr[j][i]*pow(beta,i)*(T[j]*pow(K[j][i],alpha)- X[j][i]);
				//	cout << j<<i<<" " << (T[j]*pow(K[j][i],alpha)- X[j][i]) << endl;
				}
				//qstat   << "V = "<< V << endl;
				//cout << mm<< ". V = "<< V << endl;

			//cout << "	PI" <<  endl;
			double PI = 0;
			double qq;
			for(int j=0;j<SP1;j++)
				for(int i=max(j-1,0);i<SA1;i++)
				{	qq = 0;
					if(j==i || j==i)
						qq = Q[j];
					if(j-1==i)
						qq = Q[j-1];
					PI = PI + pr[j][i]*pow(beta,i)*(X[j][i] - w_tilde*K[j][i] - Cf(K[j][i],P[j][i]) - qq);
					//	cout << j<<i<<": " <<qq<< endl;
				}

			//	cout <<"PI = " << PI << endl;

			double ICC[N1];
			//double LHS[N1];
			//double RHS[N1];
			double minICCc = MM;

			for(int n=0;n<SP1-1;n++)
			{
				ICC[n]= 0;
					
				for(int i=n;i<SA1;i++)
				{
					double prob=1.0;
					for(int it=n;it<i+1;it++)
						prob = prob*(1.0-P[n][it]);
								
					for(int j=n+1;j<min(SP1,i+2);j++)
					{	ICC[n] = ICC[n] + (pr[j][i]*pow(beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i])));
						//cout <<"ICC"<<n<<": " << j << ","<<i<<" = " << (pow(pr[j][i]*beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i]))) << endl; 		
					}	
				}	
					
				if(ICC[n]<minICCc)
					minICCc = ICC[n];

				//	cout <<"ICC"<< n <<": " << ICC[n] << endl;
			}
									
			double LL[SP1-1];
			double minLLc = MM;
			for(int n=0;n<SP1-1;n++)
			{	LL[n]= T[n+1]*pow(K[n+1][n],alpha)-X[n+1][n];
				//qstat   << "LLC"<< n << " = " << LL[n] << endl;
			////		cout << "LLC"<< n << " = " << LL[n] << endl;
				if(LL[n]<minLLc)
					minLLc = LL[n];
			}
			///*
			
			double VVaux = V - abs(PI)*AD-maxi(-minICCc,0)*AD- maxi(-minLLc,0)*AD;
				
			if(VVaux>VVmax)
			{	
					for(int ix=0;ix<C1;ix++)
					{	xgstar[ix]=xguess[ix];
						xlbstar[ix] = xlb[ix];
						xubstar[ix] =  xub[ix]; 
						xscalestar[ix]=xscale[ix];
					}
					for(int ix=0;ix<C1;ix++)
						xguess[ix] = xguess[ix]*.99;//5+.005*x[ix];
				
			}		
			else
			{
					
				double random = rand()*2.0 /RAND_MAX - 1.0;
				if(VVaux<0)
				{
					cout << "-";
				
					/* Creating a new guess */
					for(int i=0;i<(SP1-1)*SA1-(1+2+3+4+5+6+7);i++)
						xguess[i] = xguess[i]*.95*(1+random/80) ;
					
					// bounds and guess on uboundital
					for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7);i<(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1;i++)
						xguess[i] = xguess[i]*.95*(1+random/80);
					
						// bounds and guess on payments
					for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1; i< (SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1 + SP1-1;i++)
						xguess[i] = xguess[i]*.95*(1+random/80);


					for(int i=0;i<(SP1-1)*SA1-(1+2+3+4+5+6+7);i++)
					{	xlb[i] = 0;
						xub[i] = xub[i]/1.25;
						xscale[i] = xscale[i] /1.25;
					}
				// bounds and guess on uboundital
					for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7);i<(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1;i++)
					{	xlb[i] =  0;
						xub[i] = xub[i]/1.25;	//write the first best!!!!
						xscale[i]=xscale[i]/1.25;
					}
						
					// bounds and guess on payments
					for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1; i< (SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1 + SP1-1;i++)
					{	xlb[i] = xlb[i]/1.25;
						xub[i] =  xub[i]/1.25; 
						xscale[i]=xscale[i]/1.25;
					}
					
				}
				
				if(VVaux>0 && VVaux<.8*VVmax)
				{
					cout << "c";
			
					/* Creating a new guess */
					for(int i=0;i<(SP1-1)*SA1-(1+2+3+4+5+6+7);i++)
						xguess[i] = xguess[i]*.97 +random/200;
					
					// bounds and guess on uboundital
					for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7);i<(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1;i++)
						xguess[i] = xguess[i]*.97+random/200;
					
						// bounds and guess on payments
					for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1; i< (SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1 + SP1-1;i++)
						xguess[i] = xguess[i]*.97+random/200;


					for(int i=0;i<(SP1-1)*SA1-(1+2+3+4+5+6+7);i++)
					{	xlb[i] = 0;
						xub[i] = .99999999999999;
						xscale[i] = xscale[i] /1.01;
					}
				// bounds and guess on uboundital
					for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7);i<(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1;i++)
					{	xlb[i] =  0;
						xub[i] = xub[i]/1.05;	//write the first best!!!!
						xscale[i]=xscale[i]/1.01;
					}
						

					// bounds and guess on payments
					for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1; i< (SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1 + SP1-1;i++)
					{	xlb[i] = xlb[i]/1.05;
						xub[i] =  xub[i]/1.05; 
						xscale[i]=xscale[i]/1.01;
					}
					
				}
			
				if(VVaux>=VVaux*.8 && VVaux<VVmax)
				{
					cout << "v";

					for(int i=0;i<(SP1-1)*SA1-(1+2+3+4+5+6+7);i++)
						xguess[i] = xguess[i]*.7+.2*x[i]+random/400;// xguess[i] = share*xub[i]; //
					// K
					for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7);i<(SP1-1)*SA1-(1+2+3+4+5+6+7) + SP1-1;i++)
						xguess[i] = xguess[i]*.7+.2*x[i]+random/400;
					// X
					for(int i=(SP1-1)*SA1-(1+2+3+4+5+6+7)+ SP1-1; i< (SP1-1)*SA1-(1+2+3+4+5+6+7) + SP1-1 + SP1-1;i++)
						xguess[i] = xguess[i]*.7+.2*x[i]+random/400;
				}

					
			
			}
			
			if(VVaux>VVmax)
			{	cout << " ("<< VVaux << ") ";
			
				VVmax = VVaux;
			}
			imsl_free(x);
		}


		for(int ix=0;ix<C1;ix++)
				xscale[ix]=xscalestar[ix];
					

		cout << endl;
		/*RERUNNING THE WINNER*/
		x = imsl_d_constrained_nlp(fcn, T1 , 1, C1, ibtype, xlbstar, xubstar, IMSL_XGUESS, xgstar,IMSL_ITMAX, 50000, 
		/*IMSL_TAU0, 10.e-5,
		IMSL_DEL0, 10.e-8,
		IMSL_SMALLW, 10.e-8,
		IMSL_DELMIN, 10.e-8,
		IMSL_SCFMAX, 10.e-16,*/
		0);
		
		for(int j=0;j<SP1;j++)
			for(int i=0;i<SA1;i++)
					P[j][i] = 0;
			
		count = -1;
		for(int j=0;j<SP1-1;j++)
			for(int i=j;i<SA1;i++)
			{	count++;
				P[j][i] = x[count];
			//		cout << "P"<<j<<i<<" = " << P[j][i] << endl;
			}
			
		for(int i=0;i<SP1-1;i++)
		{	count++;
			K[i+1][i] = x[count];
			K[i][i] = x[count];
			//cout << "K"<<i+1<<i<<" = " << K[i+1][i] << endl;
			//cout << "K"<<i<<i<<" = " << K[i][i] << endl;
		}

		//cout << " IRREVERSABILITY " << endl;
		for(int j=0;j<SP1-1;j++)
			for(int i=j+1;i<SA1;i++)
			{	K[j][i] = K[j][j];
				//cout << "K"<<j<<i<<" = " << K[j][i] << endl;
			}

		for(int i=SP1-1;i<SA1;i++)
		{	K[SP1-1][i] = K[SP1-1][SP1-2];
			//cout << "K"<<SP1-1<<i<<" = " << K[SP1-1][i] << endl;
		}

		for(int i=0;i<SP1-1;i++)
		{	count++;
			X[i+1][i] = x[count];
		//		cout << "X"<<i+1<<i<<" = " << X[i+1][i] << endl;
		}

				double total;
			for(int j=0;j<SP1;j++)
			{	total = 0;
								for(int i=j;i<SA1;i++)
				{		X[j][i] = T[j]*pow(K[j][i],alpha);
						total = pr[j][i]*pow(beta,i-j)*T[j]*pow(K[j][i],alpha)*lambda + total;
						//X[j][i] = T[j]*pow(K[j][i],alpha)*(1.0-lambda);
						//cout << "X"<<j<<i<<" = " << X[j][i] << endl;
				}
				X[j][SA1-1] = T[j]*pow(K[j][SA1-1],alpha)-total/(pr[j][SA1-1]*pow(beta,SA1-1));
			}

		double V = 0;
		for(int j=0;j<SP1;j++)
			for(int i=max(j-1,0);i<SA1;i++)
			{	V = V + pr[j][i]*pow(beta,i)*(T[j]*pow(K[j][i],alpha)- X[j][i]);
			//	cout << j<<i<<" " << (T[j]*pow(K[j][i],alpha)- X[j][i]) << endl;
			}
		//qstat   << "V = "<< V << endl;
		//cout << "V = "<< V << endl;
		//cout << "	PI" <<  endl;

		double PI = 0;
		double qq;
		for(int j=0;j<SP1;j++)
			for(int i=max(j-1,0);i<SA1;i++)
			{	qq = 0;
				if(j==i || j==i)
					qq = Q[j];
				if(j-1==i)
					qq = Q[j-1];
				PI = PI + pr[j][i]*pow(beta,i)*(X[j][i] - w_tilde*K[j][i] - Cf(K[j][i],P[j][i]) - qq);
				//cout << j<<i<<": " <<(pr[j][i]*pow(beta,i)*(X[j][i] - w_tilde*K[j][i] - qq - Cf(K[j][i],P[j][i]) ))<< endl;
			}

		//	cout <<"PI = " << PI << endl;

		double ICC[N1];
		//double LHS[N1];
		//double RHS[N1];
		double minICCc = MM;

		for(int n=0;n<SP1-1;n++)
		{
			ICC[n]= 0;
				
			for(int i=n;i<SA1;i++)
			{
				double prob=1.0;
				for(int it=n;it<i+1;it++)
					prob = prob*(1.0-P[n][it]);
							
				for(int j=n+1;j<min(SP1,i+2);j++)
				{	ICC[n] = ICC[n] + (pr[j][i]*pow(beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i])));
					//cout <<"ICC"<<n<<": " << j << ","<<i<<" = " << (pow(pr[j][i]*beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i]))) << endl; 		
				}	
			}	
				
			if(ICC[n]<minICCc)
				minICCc = ICC[n];

			//		cout <<"ICC"<< n <<": " << ICC[n] << endl;
		}

		double LL[SP1-1];
		double minLLc = MM;
		for(int n=0;n<SP1-1;n++)
		{	LL[n]= T[n+1]*pow(K[n+1][n],alpha)-X[n+1][n];
				//qstat   << "LLC"<< n << " = " << LL[n] << endl;
				//cout << "LLC"<< n << " = " << LL[n] << endl;
			if(LL[n]<minLLc)
				minLLc = LL[n];
		}
		
		V = V - abs(PI)*AD -maxi(-minICCc,0)*AD - maxi(-minLLc,0)*AD;
			
		cout << "V = " << V << endl;
			
		if(V>=VVMAX)
		{	VVMAX = V;
			for(int ix=0;ix<C1;ix++)
			{	xgS[ix]=xgstar[ix];
				xlbG[ix] = xlbstar[ix];
				xubG[ix] = xubstar[ix];
			}
			
			//cout << "New optimizer"<<endl;

		}
			
		for(int i=0; i< C1;i++)
			xgstar[i] = xgS[i];

			
	}

	/* RERUNNING OPTIMIATION TO OBTAIN THE WINNER */	
	//imsl_omp_options(IMSL_SET_FUNCTIONS_THREAD_SAFE, 1, 0);
	x = imsl_d_constrained_nlp(fcn, T1 , 1, C1, ibtype, xlbG, xubG, IMSL_XGUESS, xgS,IMSL_ITMAX, 50000, 
			/*IMSL_TAU0, 10.e-5,
			IMSL_DEL0, 10.e-8,
			IMSL_SMALLW, 10.e-8,
			IMSL_DELMIN, 10.e-8,
			IMSL_SCFMAX, 10.e-16,*/
			0);
		

	for(int j=0;j<SP1;j++)
		for(int i=0;i<SA1;i++)
			P[j][i] = 0;
		
	int count = -1;
	for(int j=0;j<SP1-1;j++)
		for(int i=j;i<SA1;i++)
		{	count++;
			P[j][i] = x[count];
			//if(P[j][i]>1.0e-05)
			qstat << "P"<<j<<i+1<<" = " << P[j][i] << endl;
		//	cout << "P"<<j<<i+1<<" = " << P[j][i] << endl;
		}
		
	for(int i=0;i<SP1-1;i++)
	{	count++;
		K[i+1][i] = x[count];
		K[i][i] = x[count];
		//qstat << "K"<<i+1<<i+1<<" = " << K[i+1][i] << endl;
	//	cout << "K"<<i+1<<i+1<<" = " << K[i+1][i] << endl;
		//qstat << "K"<<i<<i+1<<" = " << K[i][i] << endl;
	}

	//cout << " IRREVERSABILITY " << endl;
	for(int j=0;j<SP1-1;j++)
		for(int i=j+1;i<SA1;i++)
		{	K[j][i] = K[j][j];
		//	qstat << "K"<<j<<i+1<<" = " << K[j][i] << endl;
		}

	for(int i=SP1-1;i<SA1;i++)
	{	K[SP1-1][i] = K[SP1-1][SP1-2];
	//	qstat << "K"<<SP1-1<<i+1<<" = " << K[SP1-1][i] << endl;
	}

	for(int i=0;i<SP1-1;i++)
	{	count++;
		X[i+1][i] = x[count];
//		qstat << "X"<<i+1<<i+1<<" = " << X[i+1][i] << endl;
	}

		double total;
			for(int j=0;j<SP1;j++)
			{	total = 0;
								for(int i=j;i<SA1;i++)
				{		X[j][i] = T[j]*pow(K[j][i],alpha);
						total = pr[j][i]*pow(beta,i-j)*T[j]*pow(K[j][i],alpha)*lambda + total;
						//X[j][i] = T[j]*pow(K[j][i],alpha)*(1.0-lambda);
						//cout << "X"<<j<<i<<" = " << X[j][i] << endl;
				}
				X[j][SA1-1] = T[j]*pow(K[j][SA1-1],alpha)-total/(pr[j][SA1-1]*pow(beta,SA1-1));
			}

	double V = 0;
	for(int j=0;j<SP1;j++)
	{	double nivel = V;
		for(int i=max(j-1,0);i<SA1;i++)
		{	V = V + pr[j][i]*pow(beta,i)*(T[j]*pow(K[j][i],alpha)- X[j][i]);
		//	cout << j<<i<<" " << (T[j]*pow(K[j][i],alpha)- X[j][i]) << endl;
		}

	//	qstat   << "V"<<j<<" = "<< V-nivel << endl;
			
	}
		
	//qstat   << "V = "<< V << endl;
	
	double PI = 0;
	double qq;
	for(int j=0;j<SP1;j++)
	{
		double nivel = PI;
		for(int i=max(j-1,0);i<SA1;i++)
		{	qq = 0;
			if(j==i || j==i)
				qq = Q[j];
			if(j-1==i)
				qq = Q[j-1];
			PI = PI + pr[j][i]*pow(beta,i)*(X[j][i] - w_tilde*K[j][i] - Cf(K[j][i],P[j][i]) - qq);
			//cout << j<<i<<": " <<(pr[j][i]*pow(beta,i)*(X[j][i] - w_tilde*K[j][i] - qq - Cf(K[j][i],P[j][i]) ))<< endl;
		}
	//	qstat <<"PI"<<j<<" = " << PI-nivel << endl;
	}

	//qstat <<"PI = " << PI << endl;

	double ICC[N1];
	double minICCc = MM;

	for(int n=0;n<SP1-1;n++)
	{
		ICC[n]= 0;
			
		for(int i=n;i<SA1;i++)
		{
			double prob=1.0;
			for(int it=n;it<i+1;it++)
				prob = prob*(1.0-P[n][it]);
						
			for(int j=n+1;j<min(SP1,i+2);j++)
			{	ICC[n] = ICC[n] + (pr[j][i]*pow(beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i])));
			//	cout <<"ICC"<<n<<": " << j << ","<<i<<" = " << (pow(pr[j][i]*beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i]))) << endl; 		
			}	
		}	
			
		if(ICC[n]<minICCc)
			minICCc = ICC[n];

//		qstat <<"ICC"<< n <<"=" << ICC[n] << endl;
	}
							
	double LL[SP1-1];
	double minLLc = MM;
	for(int n=0;n<SP1-1;n++)
	{	LL[n]= T[n+1]*pow(K[n+1][n],alpha)-X[n+1][n];
	//	qstat   << "LLC"<< n << " = " << LL[n] << endl;
		//cout << "LLC"<< n << " = " << LL[n] << endl;
		if(LL[n]<minLLc)
			minLLc = LL[n];
	}
	

	V = V - abs(PI)*AD -maxi(-minICCc,0)*AD - maxi(-minLLc,0)*AD;
	qstat << "V = "<< V << endl;
	cout << "Final value of	V = "<< V << endl;
	VVV = V;
	///*
	if(V>0)
	{
		ofstream out ("guess.dat");
		for(int i=0;i<C1;i++)
		{		out << x[i] << endl;
		//		//cout << xguess[i] << endl;
		}
	}
	//*/

	cout << endl;

	if(VVV>0)
	{
		double yon = 0;
		double totest = 0;
		double totmea = 0;
		
		
		double totout = 0;
		double totin = 0;
		double totlab = 0;
		double totcap = 0;
		double totlabm = 0;
		double totcapm = 0;
		double totmoni = 0;
		double totx = 0;
		double totfixed = 0;
		double totret = 0;

		
			
		double totprofits = 0;
		qstat << "j	i	prob	W	L	qk	k~"<<endl;
		
		for(int j=0;j<SP1;j++)
			for(int i=max(j-1,0);i<SA1;i++)
			{	totout  = totout + pr[j][i]*T[j]*pow(K[j][i],alpha);
				totret  = totret + pr[j][i]*( T[j]*pow(K[j][i],alpha) - X[j][i]);
				totest  = totest  + pr[j][i];
				yon   = yon  + pr[j][i]*T[j]*pow(K[j][i],alpha)/(K[j][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), -omega));
				totin = totin + pr[j][i]*K[j][i];
				totx    = totx  + pr[j][i]*X[j][i];	
				totmea = totmea + pr[j][i];
				totlab = totlab + pr[j][i]*K[j][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), -omega) ;
				totcap = totcap + pr[j][i]*K[j][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), 1-omega) ;
				
				totlabm = totlabm + pr[j][i]*(Cf(K[j][i],P[j][i])/w_tilde)*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), -omega) ;
				totcapm = totcapm + pr[j][i]*(Cf(K[j][i],P[j][i])/w_tilde)*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), 1-omega) ;
				
				totmoni    = totmoni  + pr[j][i]*(Cf(K[j][i],P[j][i]));	
		
				qq = 0;
				if(j==i || j==i)
					qq = Q[j];
				if(j-1==i)
					qq = Q[j-1];
				
				totprofits    = totprofits  + pr[j][i]*( T[j]*pow(K[j][i],alpha) - (w_tilde*K[j][i] + Cf(K[j][i],P[j][i]) + qq));	
				totfixed    = totfixed  + pr[j][i]*( qq);

				qstat << j <<"	"<< i <<"	"<< pr[j][i]<<"	"<< pr[j][i]*pow(beta,i)*(T[j]*pow(K[j][i],alpha)) <<"	"<<pr[j][i]*pow(beta,i)*(X[j][i])<<"	"<<w_tilde*K[j][i]<<"	"<<K[j][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), 1-omega)<<endl;
				
			}
		qstat <<	"o  = " << totout    << endl;
		qstat <<	"k  = " << totin     << endl;
		qstat <<	"l  = " << totlab    << endl;
		qstat <<	"k~ = " << totcap    << endl;
		qstat <<	"lm = " << totlabm   << endl;
		qstat <<	"k~m= " << totcapm   << endl;
		qstat <<	"x  = " << totx      << endl;
		qstat <<	"C  = " << totmoni   << endl;
		qstat <<	"phi= " << totfixed  << endl;
		qstat <<	"w  = " << w         << endl;
		qstat <<	"r  = " << r         << endl;
		qstat <<	"q  = " << w_tilde   << endl;
		qstat <<	"ret  = " << totret << endl;
			
		double Xn[SP1][SA1];
		for(int j=0;j<SP1;j++)
			for(int i=max(j-1,0);i<SA1;i++)
				Xn[j][i]= X[j][i]-w_tilde*K[j][i];
		
		double DJn[SP1][SA1];

		for(int j=0;j<SP1;j++)
				DJn[j][SA1-1]=Xn[j][SA1-1];
		
		for(int j=0;j<SP1;j++)
			for(int i=SA1-2; i>=max(j-1,0); i--)
				DJn[j][i]= Xn[j][i]+DJn[j][i+1]*pr[j][i+1]*beta/pr[j][i];

		for(int j=SP1-3;j>=0;j--)
			DJn[j+1][j]= Xn[j+1][j]+beta*DJn[j+1][j+1]*pr[j+1][j+1]/pr[j+1][j]+beta*DJn[j+2][j+1]*pr[j+2][j+1]/pr[j+1][j];

		double totdeudaJn = 0;
		for(int j=0;j<SP1;j++)
			for(int i=max(j-1,0);i<SA1;i++)
				totdeudaJn = totdeudaJn + pr[j][i]*max(DJn[j][i],0.0);

		double modelmean = totlab/totest;
		
		qstat << " V = " << V <<  endl;
		qstat << "Debt-to-output = " << totdeudaJn/totout << endl;
		qstat << "Profit-to-output = " << totprofits/totout  << endl;
		qstat << "Fixed-cost-to-Output = " << totfixed/totout << endl;
		qstat << "Output per worker = " << totout/totlab << endl;
		qstat << "Measured TFP = " << totout/(pow(totlab,1-omega)*pow(totcap,omega)) << endl;        // notice here I am not including A
		qstat << "SIZE = "<< modelmean << endl; 
		qstat << "K to L ratio = " <<totcap/totlab << endl;  // notice here I am not including A
		qstat << "K to Y ratio = " <<totcap/totout << endl;  // notice here I am not including A
		qstat << "mean yon = " << yon << endl;  
		qstat << "number workers = " << totlab << endl;
		qstat << "MonCosts-to-Output = " << totmoni/totout << endl;
		qstat << "Wages = " << w << endl;
		qstat << "Wages by efficiency units = " << w/A << endl;
		qstat << "Cost Amalgamated input = " << w_tilde << endl;


		
	// HERE I HAVE TO COMPUTE AGGREGA8TE STATISTICS


	/*	double VALUE= 0;
		double DEBT= 0;
		double ASSETS = 0;


		qstat   << "Value	Debt	Assets	"<<endl;
			
		for(int i=SA1-1;i>-1;i--)
		{	
			for(int j=0;j<SP1;j++)
			{	
				VALUE  = VALUE + pr[j][i]*pow(beta,i)*(T[j]*pow(K[j][i],alpha)- w_tilde*K[j][i] - Cf(K[j][i],P[j][i]));
				DEBT   = DEBT  + pr[j][i]*pow(beta,i)*(X[j][i]);
				ASSETS = ASSETS + pr[j][i]*pow(beta,i)*T[j]*pow(K[j][i],alpha);
			}
			
			qstat   <<i <<"	"<< VALUE << "	"<<DEBT << "	"<< ASSETS << endl;
				
		}
			*/

//		qstat   << "V = "<< V << endl;
		
		double PI = 0;
		double qq;
		for(int j=0;j<SP1;j++)
		{
			double nivel = PI;
			for(int i=max(j-1,0);i<SA1;i++)
			{	qq = 0;
				if(j==i || j==i)
					qq = Q[j];
				if(j-1==i)
					qq = Q[j-1];
				PI = PI + pr[j][i]*pow(beta,i)*(X[j][i] - w_tilde*K[j][i] - Cf(K[j][i],P[j][i]) - qq);
				//cout << j<<i<<": " <<(pr[j][i]*pow(beta,i)*(X[j][i] - w_tilde*K[j][i] - qq - Cf(K[j][i],P[j][i]) ))<< endl;
			}
	//		qstat <<"PI"<<j<<" = " << PI-nivel << endl;
		}

	//	qstat <<"PI = " << PI << endl;

		double ICC[N1];
		double minICCc = MM;

		for(int n=0;n<SP1-1;n++)
		{
			ICC[n]= 0;
				
			for(int i=n;i<SA1;i++)
			{
				double prob=1.0;
				for(int it=n;it<i+1;it++)
					prob = prob*(1.0-P[n][it]);
							
				for(int j=n+1;j<min(SP1,i+2);j++)
				{	ICC[n] = ICC[n] + (pr[j][i]*pow(beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i])));
				//	cout <<"ICC"<<n<<": " << j << ","<<i<<" = " << (pow(pr[j][i]*beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i]))) << endl; 		
				}	
			}	
				
			if(ICC[n]<minICCc)
				minICCc = ICC[n];

	//		qstat <<"ICC"<< n <<"=" << ICC[n] << endl;
		}



		
		//qstat << endl;

		double output[SA1];
		for(int i=0;i<SA1;i++)
			output[i]=0;

		for(int j=0;j<SP1;j++)
			for(int i=max(j-1,0);i<SA1;i++)
			{	output[i]  = output[i]  + pr[j][i]*T[j]*pow(K[j][i],alpha);
			}

	//	cout << "output = " << totout << endl;

//		for(int i=0;i<SA1;i++)
//			qstat << i+1 << "Output = " << 100*output[i]/totout << endl;

	//	qstat << endl;

		double estab[SA1];
		for(int i=0;i<SA1;i++)
			estab[i]=0;


		for(int j=0;j<SP1;j++)
			for(int i=max(j-1,0);i<SA1;i++)
			{	estab[i]  = estab[i]  + pr[j][i];
			}





		double input[SA1];
		double empl[SA1];
		double va[SA1];
		double cap[SA1];
		double tfp[SA1];
		double est[SA1];
		double wtfp[SA1];
		double xss[SA1];
		double fixed[SA1];
		double moni[SA1];
		double moniPR[SA1];
		double WKmoni[SA1];
		double WK[SA1];
		double debtLC[SA1];

		for(int i=0;i<SA1;i++)
		{	input[i]=0;
			empl[i]=0;
			va[i]=0;
			cap[i]=0;
			tfp[i] =0;
			wtfp[i] =0;
			est[i] =0;
			xss[i]=0;
			fixed[i] =0;
			moni[i]=0;
			moniPR[i]=0;
			WKmoni[i]=0;
			WK[i]=0;
			debtLC[i];

		}


		
		int types = 0;

		for(int j=0;j<SP1;j++)
			for(int i=max(j-1,0);i<SA1;i++)
			{	
				qq = 0;
				if(j==i || j==i)
					qq = Q[j];
				if(j-1==i)
					qq = Q[j-1];

				input[i]  = input[i]  + pr[j][i]*K[j][i];
				xss[i]    = xss[i]  + pr[j][i]*(w_tilde*K[j][i] + Cf(K[j][i],P[j][i]) + qq);	

				fixed[i]    = fixed[i]  + pr[j][i]*( qq);	
				
				moni[i]    = moni[i]  + pr[j][i]*(Cf(K[j][i],P[j][i]));	
				
				moniPR[i]  = moniPR[i] + pr[j][i]*P[j][i];	
				
				WKmoni[i]    = WKmoni[i]  + pr[j][i]*(Cf(K[j][i],P[j][i]))/w_tilde;	

				WK[i]    = WK[i]  + pr[j][i]*K[j][i];	


				empl[i]   = empl[i]  + pr[j][i]*K[j][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), -omega) ;
				va[i]     = va[i]  +             pr[j][i]*T[j]*pow(K[j][i],alpha);
				tfp[i]    = tfp[i]  + pr[j][i]*pow(T[j],2.0);
				est[i]    = est[i]  + pr[j][i];
				cap[i]    = cap[i]                   + pr[j][i]*K[j][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), 1-omega) ;

				wtfp[i]    = wtfp[i]  + T[j]*pr[j][i]*T[j]*pow(K[j][i],alpha);
				
				debtLC[i] = debtLC[i] + pr[j][i]*max(DJn[j][i],0.0);

				types++;
			}

		
		qstat << "AGE	debtTOoutput"<< endl;
		qstat << "<5	" << debtLC[0]/output[0]<<endl;
		qstat << "5-10	" << debtLC[1]/output[1]<<endl;
		qstat << "10-15	" << debtLC[2]/output[2]<<endl;
		qstat << "15-20	" << debtLC[3]/output[3]<<endl;
		qstat << "20-25	" << debtLC[4]/output[4]<<endl;
		qstat << "25-30	" << debtLC[5]/output[5]<<endl;
		qstat << "30-35	" << debtLC[6]/output[6]<<endl;
		qstat << "35-40	" << debtLC[7]/output[7]<<endl;
		qstat << "40-45	" << debtLC[8]/output[8]<<endl;
		qstat << "45-50	" << debtLC[9]/output[9]<<endl;
		qstat << endl;


		qstat << "AGE	monitoring"<< endl;
		qstat << "<5	" << moniPR[0]/est[0]<<endl;
		qstat << "5-10	" << moniPR[1]/est[1]<<endl;
		qstat << "10-15	" << moniPR[2]/est[2]<<endl;
		qstat << "15-20	" << moniPR[3]/est[3]<<endl;
		qstat << "20-25	" << moniPR[4]/est[4]<<endl;
		qstat << "25-30	" << moniPR[5]/est[5]<<endl;
		qstat << "30-35	" << moniPR[6]/est[6]<<endl;
		qstat << "35-40	" << moniPR[7]/est[7]<<endl;
		qstat << "40-45	" << moniPR[8]/est[8]<<endl;
		qstat << "45-50	" << moniPR[9]/est[9]<<endl;
		qstat << endl;


		qstat << "AGE	outputTOcapital"<< endl;
		qstat << "<5	" << output[0]/cap[0]<<endl;
		qstat << "5-10	" << output[1]/cap[1]<<endl;
		qstat << "10-15	" << output[2]/cap[2]<<endl;
		qstat << "15-20	" << output[3]/cap[3]<<endl;
		qstat << "20-25	" << output[4]/cap[4]<<endl;
		qstat << "25-30	" << output[5]/cap[5]<<endl;
		qstat << "30-35	" << output[6]/cap[6]<<endl;
		qstat << "35-40	" << output[7]/cap[7]<<endl;
		qstat << "40-45	" << output[8]/cap[8]<<endl;
		qstat << "45-50	" << output[9]/cap[9]<<endl;
		qstat << endl;

	

			//	qstat << endl;
//	qstat << totx1/totx <<"	"<< totx2/totx <<"	"<< totx3/totx << endl;
//	qstat << endl;



//		qstat << endl;
//		qstat << "Debt to output = " << totdeuda/totout << endl;
/*		qstat << endl;
		qstat << "Debt1 to output = " << totdeuda1/totout << endl;
		qstat << endl;
	    qstat << "Debt2 to output = " << totdeuda2/totout << endl;
		qstat << endl;
	    qstat << "Debt3 to output = " << totdeuda3/totout << endl;
		qstat << endl;

		qstat << "DebtJ to output = " << totdeudaJ/totout << endl;
		qstat << endl;
*/
			
/*		qstat << endl;

	double IC[N1][4];

	for(int n=0;n<SP1-1;n++)
	{
		IC[n][0]= 0;
			
		for(int i=n;i<SA1;i++)
		{
			double prob=1.0;
			for(int it=n;it<i+1;it++)
				prob = prob*(1.0-P[n][it]);
						
			for(int j=n+1;j<min(SP1,i+2);j++)
			{	IC[n][0] = IC[n][0] + (pr[j][i]*pow(beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i])));
				//cout <<"ICC"<<n<<": " << j << ","<<i<<" = " << (pow(pr[j][i]*beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i]))) << endl; 		
			}	
		}	
			
		IC[n][1]= 0;
			
		for(int i=n;i<SA1;i++)
		{
			double prob=1.0;
			for(int it=n;it<i+1;it++)
				prob = prob*(1.0-P[n][it]);
						
			for(int j=n+1;j<min(SP1,i+2);j++)
			{	IC[n][1] = IC[n][1] + (pr[j][i]*pow(beta,i-n)*((T[j]*pow(K[j][i],alpha)- X1[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X1[n][i])));
				//cout <<"ICC"<<n<<": " << j << ","<<i<<" = " << (pow(pr[j][i]*beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i]))) << endl; 		
			}	
		}	
			
		IC[n][2]= 0;
			
		for(int i=n;i<SA1;i++)
		{
			double prob=1.0;
			for(int it=n;it<i+1;it++)
				prob = prob*(1.0-P[n][it]);
						
			for(int j=n+1;j<min(SP1,i+2);j++)
			{	IC[n][2] = IC[n][2] + (pr[j][i]*pow(beta,i-n)*((T[j]*pow(K[j][i],alpha)- X2[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X2[n][i])));
				//cout <<"ICC"<<n<<": " << j << ","<<i<<" = " << (pow(pr[j][i]*beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i]))) << endl; 		
			}	
		}	
			
		IC[n][3]= 0;
			
		for(int i=n;i<SA1;i++)
		{
			double prob=1.0;
			for(int it=n;it<i+1;it++)
				prob = prob*(1.0-P[n][it]);
						
			for(int j=n+1;j<min(SP1,i+2);j++)
			{	IC[n][3] = IC[n][3] + (pr[j][i]*pow(beta,i-n)*((T[j]*pow(K[j][i],alpha)- X3[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X3[n][i])));
				//cout <<"ICC"<<n<<": " << j << ","<<i<<" = " << (pow(pr[j][i]*beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i]))) << endl; 		
			}	
		}	
			
		qstat <<"IC"<< n <<"=" << IC[n][0] << "=" << IC[n][1]<< "=" << IC[n][2] <<"=" << IC[n][3] << endl;
	}
	

			
		qstat << "AGE and Debt/Output"<< endl;
		qstat << "<5	" << deuda[0]/output[0]<<endl;
		qstat << "5-10	" << deuda[1]/output[1]<<endl;
		qstat << "10-15	" << deuda[2]/output[2]<<endl;
		qstat << "15-20	" << deuda[3]/output[3]<<endl;
		qstat << "20-25	" << deuda[4]/output[4]<<endl;
		qstat << "25-30	" << deuda[5]/output[5]<<endl;
		qstat << "30-35	" << deuda[6]/output[6]<<endl;
		qstat << "35-40	" << deuda[7]/output[7]<<endl;
		qstat << "40-45	" << deuda[8]/output[8]<<endl;
		qstat << "45-50	" << deuda[9]/output[9]<<endl;
		qstat << endl;
		qstat << "Debt to output = " << totdeuda/totout << endl;
		qstat << endl;

		qstat << "AGE and Debt2/Output"<< endl;
		qstat << "<5	" << deuda2[0]/output[0]<<endl;
		qstat << "5-10	" << deuda2[1]/output[1]<<endl;
		qstat << "10-15	" << deuda2[2]/output[2]<<endl;
		qstat << "15-20	" << deuda2[3]/output[3]<<endl;
		qstat << "20-25	" << deuda2[4]/output[4]<<endl;
		qstat << "25-30	" << deuda2[5]/output[5]<<endl;
		qstat << "30-35	" << deuda2[6]/output[6]<<endl;
		qstat << "35-40	" << deuda2[7]/output[7]<<endl;
		qstat << "40-45	" << deuda2[8]/output[8]<<endl;
		qstat << "45-50	" << deuda2[9]/output[9]<<endl;
		qstat << endl;

	    qstat << "Debt2 to output = " << totdeuda2/totout << endl;
		qstat << endl;


		qstat << "AGE and Debt3/Output"<< endl;
		qstat << "<5	" << deuda3[0]/output[0]<<endl;
		qstat << "5-10	" << deuda3[1]/output[1]<<endl;
		qstat << "10-15	" << deuda3[2]/output[2]<<endl;
		qstat << "15-20	" << deuda3[3]/output[3]<<endl;
		qstat << "20-25	" << deuda3[4]/output[4]<<endl;
		qstat << "25-30	" << deuda3[5]/output[5]<<endl;
		qstat << "30-35	" << deuda3[6]/output[6]<<endl;
		qstat << "35-40	" << deuda3[7]/output[7]<<endl;
		qstat << "40-45	" << deuda3[8]/output[8]<<endl;
		qstat << "45-50	" << deuda3[9]/output[9]<<endl;
		qstat << endl;

	    qstat << "Debt3 to output = " << totdeuda3/totout << endl;
		qstat << endl;


		qstat << "AGE and Debt4/Output"<< endl;
		qstat << "<5	" << deuda4[0]/output[0]<<endl;
		qstat << "5-10	" << deuda4[1]/output[1]<<endl;
		qstat << "10-15	" << deuda4[2]/output[2]<<endl;
		qstat << "15-20	" << deuda4[3]/output[3]<<endl;
		qstat << "20-25	" << deuda4[4]/output[4]<<endl;
		qstat << "25-30	" << deuda4[5]/output[5]<<endl;
		qstat << "30-35	" << deuda4[6]/output[6]<<endl;
		qstat << "35-40	" << deuda4[7]/output[7]<<endl;
		qstat << "40-45	" << deuda4[8]/output[8]<<endl;
		qstat << "45-50	" << deuda4[9]/output[9]<<endl;
		qstat << endl;

	    qstat << "Debt4 to output = " << totdeuda4/totout << endl;*/

	//	qstat << endl;
	//	qstat << "Output = " << (va[0]+va[1]+va[2]+va[3]+va[4]+va[5]+va[6]+va[7]+va[8]+va[9])<< endl;
	//	qstat << "Total-K = " << (cap[0]+cap[1]+cap[2]+cap[3]+cap[4]+cap[5]+cap[6]+cap[7]+cap[8]+cap[9])<< endl;
	//	qstat << "Profits = " << ((va[0]+va[1]+va[2]+va[3]+va[4]+va[5]+va[6]+va[7]+va[8]+va[9])-(xss[0]+xss[1]+xss[2]+xss[3]+xss[4]+xss[5]+xss[6]+xss[7]+xss[8]+xss[9]))<< endl;
		//qstat << "Monitoring-cost-to-Output = " << (moni[0]+moni[1]+moni[2]+moni[3]+moni[4]+moni[5]+moni[6]+moni[7]+moni[8]+moni[9])/(va[0]+va[1]+va[2]+va[3]+va[4]+va[5]+va[6]+va[7]+va[8]+va[9])<< endl;		
		//qstat << "Monitoring-cost-to-WK = " << (moni[0]+moni[1]+moni[2]+moni[3]+moni[4]+moni[5]+moni[6]+moni[7]+moni[8]+moni[9])/(WK[0]+WK[1]+WK[2]+WK[3]+WK[4]+WK[5]+WK[6]+WK[7]+WK[8]+WK[9])<< endl;		
		//qstat << "WK-in-Monitoring = " << (WKmoni[0]+WKmoni[1]+WKmoni[2]+WKmoni[3]+WKmoni[4]+WKmoni[5]+WKmoni[6]+WKmoni[7]+WKmoni[8]+WKmoni[9])<< endl;
		//qstat << "Total-WK = " << (WK[0]+WK[1]+WK[2]+WK[3]+WK[4]+WK[5]+WK[6]+WK[7]+WK[8]+WK[9])<< endl;
	qstat << "Finance-Efficiency = " << (WK[0]+WK[1]+WK[2]+WK[3]+WK[4]+WK[5]+WK[6]+WK[7]+WK[8]+WK[9])/pow(WKmoni[0]+WKmoni[1]+WKmoni[2]+WKmoni[3]+WKmoni[4]+WKmoni[5]+WKmoni[6]+WKmoni[7]+WKmoni[8]+WKmoni[9],.8)<< endl;
	//	qstat << endl;

		

	//	cout << types << endl;

//		for(int i=0;i<SA1;i++)
//			qstat << i+1 << "Input = " << 100*empl[i]/totlab << endl;

		//qstat << endl;
		
		//cout << types << endl;
		//qstat << endl;

		
		//qstat << "<10 = " << (empl[0]+empl[1])/totlab << endl;
		//qstat << "10-20 = " <<  (empl[2]+empl[3])/totlab << endl;
		//qstat << "20-35 = " <<  (empl[4]+empl[5]+empl[6])/totlab << endl;
		//qstat << ">35 = " <<  (empl[7]+empl[8]+empl[9])/totlab << endl;
		
		
		double inputs[62][3];
		int ii=0;
		for(int j=0;j<SP1;j++)
		{	for(int i=max(j-1,0);i<SA1;i++)
			{	if(ii>0)
				{	inputs[ii][1]=pr[j][i]+inputs[ii-1][1];
					inputs[ii][0]=K[j][i]*pr[j][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), -omega) +inputs[ii-1][0];
					inputs[ii][2]=K[j][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), -omega) ;
				}
				else
				{	inputs[ii][1]=pr[j][i];
					inputs[ii][0]=K[j][i]*pr[j][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), -omega) ;
					inputs[ii][2]=K[j][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), -omega) ;
				}
				ii++;
			}
		}
		
		// bubble sorting
		double temp0,temp1,temp2;

		for(int i=0;i<61;i++)
		for(int j=0;j<(61-i);j++)
			if(inputs[j][2]>inputs[j+1][2])
			{ 
				temp2=inputs[j][2];
				inputs[j][2]=inputs[j+1][2];
				inputs[j+1][2]=temp2;

				temp1=inputs[j][1];
				inputs[j][1]=inputs[j+1][1];
				inputs[j+1][1]=temp1;

				temp0=inputs[j][0];
				inputs[j][0]=inputs[j+1][0];
				inputs[j+1][0]=temp0;
			}

		double share_emp[63];
		double share_est[63];
		
		share_emp[0]=0;
		share_est[0]=0;
		
		for(int i=1;i<63;i++)
		{
			share_emp[i] = inputs[i][0]/totlab;
			share_est[i] = inputs[i][1]/totest;

			//cout << inputs[i][2] << "	" << share_emp[i] <<"	" << share_est[i] << endl;
	
		}

			
		/*



		qstat << endl;

		double growth[SA1-2];
		for(int i=0;i<SA1-3;i++)
		{	double deno=0;
			for(int j=0;j<i+3;j++)
				deno = deno + pr[j][i+1];
		
			growth[i]  = pr[i+2][i+1] / (deno)         *( (K[i+2][i+1]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), -omega)) / (K[i+1][i]*(1.0/(A))*pow((w/r)*(omega/(1.0-omega)), -omega)) - 1.0);
			qstat << "Age-group" << i+1 << " growth rate = " << growth[i]*100 << endl;
			cout << "Age-group" << i+1 << " growth rate = " << growth[i]*100 << endl;
		}

		qstat << endl;*/

		/////////////////////////////////////////////
		//////////// TARGETS /////////////////////
		
		////////// INDIA ///////////////

		// Age distribution INDIA, manufacturing

		double sxA[9] = {0,5,10,15,20,25,30,35,40};
		double syA[9] = {1, 0.7184328, 0.4939829, 0.3391503, 0.2498291, 0.1831509, 0.13969, 0.1040082, 0.0861181};


		Imsl_d_ppoly *ppolyA;
		ppolyA = imsl_d_cub_spline_interp_e_cnd (9, sxA, syA, 0);		
		
		double ageL5	  =  imsl_d_cub_spline_value(5,ppolyA,0);
		double ageL10	  =  imsl_d_cub_spline_value(10,ppolyA,0);
		double ageL15	  =  imsl_d_cub_spline_value(15,ppolyA,0);
		double ageL20	  =  imsl_d_cub_spline_value(20,ppolyA,0);
		double ageL25	  =  imsl_d_cub_spline_value(25,ppolyA,0);
		double ageL30	  =  imsl_d_cub_spline_value(30,ppolyA,0);
		double ageL35	  =  imsl_d_cub_spline_value(35,ppolyA,0);
		double ageL40	  =  imsl_d_cub_spline_value(40,ppolyA,0);
		

		printf("	\n");
		printf("AGE DISTRIBUTION OF EMPLOYMENT\n");
		printf(">=	      Model	Data\n");
		printf("5  years old %4.5f  %4.5f\n", (empl[1]+empl[2]+empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab, ageL5);
		printf("10 years old %4.5f  %4.5f\n", (empl[2]+empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab, ageL10);
		printf("15 years old %4.5f  %4.5f\n", (empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab, ageL15);
		printf("20 years old %4.5f  %4.5f\n", (empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab, ageL20);
		printf("25 years old %4.5f  %4.5f\n", (empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab, ageL25);
		printf("30 years old %4.5f  %4.5f\n", (empl[6]+empl[7]+empl[8]+empl[9])/totlab, ageL30);
		printf("35 years old %4.5f  %4.5f\n", (empl[7]+empl[8]+empl[9])/totlab, ageL35);
		printf("40 years old %4.5f  %4.5f\n", (empl[8]+empl[9])/totlab, ageL40);
		printf("	\n");

		qstat << "AGE DISTRIBUTION OF EMPLOYMENT"<< endl;
		qstat << "	Model	Data"<< endl;
		qstat << "5=	" << (empl[1]+empl[2]+empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab<<"=	"<< ageL5<< endl;
		qstat << "10=	" << (empl[2]+empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab<<"=	"<< ageL10<< endl;
		qstat << "15=	" << (empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab<<"=	"<< ageL15<< endl;
		qstat << "20=	" << (empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab<<"=	"<< ageL20<< endl;
		qstat << "25=	" << (empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab<<"=	"<< ageL25<< endl;
		qstat << "30=	" << (empl[6]+empl[7]+empl[8]+empl[9])/totlab<<"=	"<< ageL30<< endl;
		qstat << "35=	" << (empl[7]+empl[8]+empl[9])/totlab<<"=	"<< ageL35<< endl;
		qstat << "40=	" << (empl[8]+empl[9])/totlab<<"=	"<<ageL40<< endl;
		qstat << endl;

		qstat << "TFP5-  =	" << tfp[0]/est[0] << endl;
		qstat << "TFP35+ =	" << (tfp[8]+tfp[9])/(est[8]+est[9])<< endl;

		qstat << "WTFP5-  =	" << wtfp[0]/va[0] << endl;
		qstat << "WTFP35+ =	" << (wtfp[8]+wtfp[9])/(va[8]+va[9])<< endl;


/*
				cout << "AGEvsMTFP"<< endl;
		cout << "<5	"     << (((va[0]/pow(empl[0],.8))/ (va[0]/pow(empl[0],.8))))<<endl;
		cout << "5-10	" << (((va[1]/pow(empl[1],.8))/ (va[0]/pow(empl[0],.8))))<<endl;
		cout << "10-15	" << (((va[2]/pow(empl[2],.8))/ (va[0]/pow(empl[0],.8))))<<endl;
		cout << "15-20	" << (((va[3]/pow(empl[3],.8))/ (va[0]/pow(empl[0],.8))))<<endl;
		cout << "20-25	" << (((va[4]/pow(empl[4],.8))/ (va[0]/pow(empl[0],.8))))<<endl;
		cout << "25-30	" << (((va[5]/pow(empl[5],.8))/ (va[0]/pow(empl[0],.8))))<<endl;
		cout << "30-35	" << (((va[6]/pow(empl[6],.8))/ (va[0]/pow(empl[0],.8))))<<endl;
		cout << "35-40	" << (((va[7]/pow(empl[7],.8))/ (va[0]/pow(empl[0],.8))))<<endl;
		cout << ">40	" << (((va[8]+va[9])/(pow(empl[8]+empl[9],.8)))/(va[0]/(pow(empl[0],.8))))<<endl;
		cout << endl;

		cout << "AGE and WTFP"<< endl;
		cout << "<5	"     << log(wtfp[0]/va[0]/(wtfp[0]/va[0]))<<endl;
		cout  << "5-10	" << log(wtfp[1]/va[1]/(wtfp[0]/va[0]))<<endl;
		cout  << "10-15	" << log(wtfp[2]/va[2]/(wtfp[0]/va[0]))<<endl;
		cout  << "15-20	" << log(wtfp[3]/va[3]/(wtfp[0]/va[0]))<<endl;
		cout  << "20-25	" << log(wtfp[4]/va[4]/(wtfp[0]/va[0]))<<endl;
		cout  << "25-30	" << log(wtfp[5]/va[5]/(wtfp[0]/va[0]))<<endl;
		cout  << "30-35	" << log(wtfp[6]/va[6]/(wtfp[0]/va[0]))<<endl;
		cout  << "35-40	" << log(wtfp[7]/va[7]/(wtfp[0]/va[0]))<<endl;
		cout  << ">40	" << log((wtfp[8]+wtfp[9])/(va[8]+va[9])/(wtfp[0]/va[0]))<<endl;
		cout  << endl;


		qstat << "AGE and WTFP"<< endl;
		qstat << "<5	"     << log(wtfp[0]/va[0]/(wtfp[0]/va[0]))<<endl;
		qstat << "5-10	" << log(wtfp[1]/va[1]/(wtfp[0]/va[0]))<<endl;
		qstat << "10-15	" << log(wtfp[2]/va[2]/(wtfp[0]/va[0]))<<endl;
		qstat << "15-20	" << log(wtfp[3]/va[3]/(wtfp[0]/va[0]))<<endl;
		qstat << "20-25	" << log(wtfp[4]/va[4]/(wtfp[0]/va[0]))<<endl;
		qstat << "25-30	" << log(wtfp[5]/va[5]/(wtfp[0]/va[0]))<<endl;
		qstat << "30-35	" << log(wtfp[6]/va[6]/(wtfp[0]/va[0]))<<endl;
		qstat << "35-40	" << log(wtfp[7]/va[7]/(wtfp[0]/va[0]))<<endl;
		qstat << ">40	" << log((wtfp[8]+wtfp[9])/(va[8]+va[9])/(wtfp[0]/va[0]))<<endl;
		qstat << endl;

*/


/*
		// Lorenz curve,  INDIA 2002, manufacturing
		double sx[6] = {0,0.786,0.960,0.991, 0.995,1};
		double sy[6] = {0,0.439,0.646,0.705, 0.761,1};

		//Imsl_d_ppoly *ppoly;
		Imsl_d_spline *ppoly;
		//ppoly = imsl_d_cub_spline_interp_e_cnd (11, sx, sy, 0);		
		ppoly = imsl_d_spline_interp(6, sx, sy,IMSL_ORDER, 2, 0);
		
		double Lorenz50d = imsl_d_cub_spline_value(0.5,ppoly,0);
		double Lorenz70d = imsl_d_cub_spline_value(0.7,ppoly,0);
		double Lorenz90d = imsl_d_cub_spline_value(0.9,ppoly,0);
		double Lorenz95d = imsl_d_cub_spline_value(0.95,ppoly,0);
		double Lorenz97d = imsl_d_cub_spline_value(0.97,ppoly,0);
		double Lorenz99d = imsl_d_cub_spline_value(0.99,ppoly,0);
		double Lorenz995d = imsl_d_cub_spline_value(0.995,ppoly,0);

		double Lorenz50d = imsl_d_spline_value(0.78,ppoly,0);
		double Lorenz70d = imsl_d_spline_value(0.96,ppoly,0);
		double Lorenz90d = imsl_d_spline_value(0.99,ppoly,0);
		double Lorenz95d = imsl_d_spline_value(0.995,ppoly,0);
*/		
//		qstat << "Size	estab	empl"<< endl;
//		for(int j=1;j<63;j++)
//			qstat << inputs[j][2]<<"	"<< share_est[j] <<"	" << share_emp[j] << endl;


		Imsl_d_ppoly *ppolyM;
		//Imsl_d_spline *ppolyM;
		ppolyM = imsl_d_cub_spline_interp_e_cnd (63, share_est, share_emp, 0);		
		//ppolyM = imsl_d_spline_interp (63, share_est, share_emp,IMSL_ORDER, 2, 0);		
		
		double Lorenz50m = imsl_d_cub_spline_value(0.5,ppolyM,0);
		double Lorenz70m = imsl_d_cub_spline_value(0.7,ppolyM,0);
		double Lorenz90m = imsl_d_cub_spline_value(0.9,ppolyM,0);
		double Lorenz95m = imsl_d_cub_spline_value(0.95,ppolyM,0);
		double Lorenz97m = imsl_d_cub_spline_value(0.97,ppolyM,0);
		double Lorenz99m = imsl_d_cub_spline_value(0.99,ppolyM,0);
		double Lorenz995m = imsl_d_cub_spline_value(0.995,ppolyM,0);

//		double Lorenz50m = imsl_d_spline_value(0.78,ppolyM,0);
//		double Lorenz70m = imsl_d_spline_value(0.96,ppolyM,0);
//		double Lorenz90m = imsl_d_spline_value(0.99,ppolyM,0);
//		double Lorenz95m = imsl_d_spline_value(0.995,ppolyM,0);


		qstat<< endl;
		qstat<< "SIZE DISTRIBUTION OF EMPLOYMENT"<< endl;
		qstat<< "Smallest       Model"<< endl;
		qstat<< "50	"<<Lorenz50m << endl;
		qstat<< "70	"<<Lorenz70m << endl;
		qstat<< "90	"<<Lorenz90m << endl;
		qstat<< "95	"<<Lorenz95m << endl;
		qstat<< "97	"<<Lorenz97m << endl;
		qstat<< "99	"<<Lorenz99m << endl;
		qstat<< "995"<<Lorenz995m << endl;
		qstat<< endl;

/*		
		double distance_age = abs((empl[1]+empl[2]+empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL5)
							+ abs((empl[2]+empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL10)
							+ abs((empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL15)
							+ abs((empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL20)
							+ abs((empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL25)
							+ abs((empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL30)
							+ abs((empl[7]+empl[8]+empl[9])/totlab-ageL35)
							+ abs((empl[8]+empl[9])/totlab-ageL40);

	
		double distance_size = abs(Lorenz50m - Lorenz50d)+0.5*abs(Lorenz70m - Lorenz70d)+0.2*abs(Lorenz90m - Lorenz90d) + 0.05*abs(Lorenz95m - Lorenz95d);

		double relative_out = totout/totlab/122705;
		cout << "Output per worker = " << totout/totlab/122705 << endl;
		cout << "K to Y ratio = " <<totout/totcap << endl;  // notice here I am not including A

		cout << "output = " << totout << endl;
		cout << "# wkrs = " << totlab << endl;

		double relative_size= modelmean/9.98583e-007;
			

		cout << "MEAN SIZE OF ESTABLISHMENTS = "<< modelmean/9.98583e-007<< endl; 
//		printf("%4.5f	  Age distance  \n", distance_age);

//		printf("%4.5f   Size distance \n", distance_size);
//		printf("%4.5f   Squared distance \n", distance2);
//		printf("\n", distance2);

				
		double distance2  = 2.25*abs((empl[1]+empl[2]+empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL5)
						  + 2.0*abs((empl[2]+empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL10)
						  + 1.75*abs((empl[3]+empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL15)
						  +	1.5*abs((empl[4]+empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL20)
						  +	1.5*abs((empl[5]+empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL25)
						  + 1.75*abs((empl[6]+empl[7]+empl[8]+empl[9])/totlab-ageL30)
						  + 2.0*abs((empl[7]+empl[8]+empl[9])/totlab-ageL35)
					  	  + 2.0*abs((empl[8]+empl[9])/totlab-ageL40)
						  + 1.5*abs(Lorenz50m - Lorenz50d)
						  + 1.5*abs(Lorenz70m - Lorenz70d)
						  + 2.0*abs(Lorenz90m - Lorenz90d) 
						  + 1.5*abs(Lorenz95m - Lorenz95d);


		qstat<< search_count <<" ubound = " << ubound <<" PHI = " << Q[0]<<" z= " << z <<" = " << VVV/totout<<" = "<<	th_c<<" = "<<  th_a<<" = "<<	 th_a2<<" = "<<	th_a3<<" = "<<	 rho_slope <<" = "<<rho_intercept <<" = " << distance_age<<" = "<<	distance_size<< " = "<<	distance2<< endl;
		cout << search_count <<" ubound = " << ubound <<" PHI = " << Q[0] <<" z= "<< z <<" th_c= "<<	th_c<<", th_a= "<<  th_a<<" th_a2= "<<	 th_a2<<" th_a3= "<<	th_a3<< endl;
		search_count ++;
		search_count ++;
		printf("A = %4.5f\n", A);
		printf("w = %4.5f\n", w);
		printf("r = %4.5f\n", r);
		printf("q = %4.5f\n", w_tilde);
		printf("h = %4.5f\n", al);
*/

	}

	

rho_slope= rho_slope+0.00065;				// exponent in theta profile	
}	

rho_intercept = rho_intercept+0.005;				// exponent in theta profile
}

th_a3= th_a3+0.00005;				// exponent in theta profile
}	

th_a2= th_a2+0.0005;				// exponent in theta profile
}

th_a= th_a+0.005;				// slope in theta profile
}

th_c= th_c+.005;				// constant in theta profile
}
		

	return 0;

}


void fcn(int n, double x[], int iact, double *result, int *ierr)
{

	for(int j=0;j<SP1;j++)
		for(int i=0;i<SA1;i++)
			P[j][i] = 0;
	
	int count = -1;
	for(int j=0;j<SP1-1;j++)
		for(int i=j;i<SA1;i++)
		{	count++;
			P[j][i] = x[count];
//			cout << "P"<<j<<i<<" = " << P[j][i] << endl;
		}
	
	for(int i=0;i<SP1-1;i++)
	{	count++;
		K[i+1][i] = x[count];
		K[i][i] = x[count];
//		cout << "K"<<i+1<<i<<" = " << K[i+1][i] << endl;
//		cout << "K"<<i<<i<<" = " << K[i][i] << endl;
	}

	//cout << " IRREVERSABILITY " << endl;

	for(int j=0;j<SP1-1;j++)
		for(int i=j+1;i<SA1;i++)
		{	K[j][i] = K[j][j];
//			cout << "K"<<j<<i<<" = " << K[j][i] << endl;
		}


	for(int i=SP1-1;i<SA1;i++)
	{	K[SP1-1][i] = K[SP1-1][SP1-2];
//		cout << "K"<<SP1-1<<i<<" = " << K[SP1-1][i] << endl;
	}

	//double nada;
	//cin >> nada;

	for(int i=0;i<SP1-1;i++)
	{	count++;
		X[i+1][i] = x[count];
//		cout << "X"<<i+1<<i<<" = " << X[i+1][i] << endl;
	}


		double total;
			for(int j=0;j<SP1;j++)
			{	total = 0;
								for(int i=j;i<SA1;i++)
				{		X[j][i] = T[j]*pow(K[j][i],alpha);
						total = pr[j][i]*pow(beta,i-j)*T[j]*pow(K[j][i],alpha)*lambda + total;
						//X[j][i] = T[j]*pow(K[j][i],alpha)*(1.0-lambda);
						//cout << "X"<<j<<i<<" = " << X[j][i] << endl;
				}
				X[j][SA1-1] = T[j]*pow(K[j][SA1-1],alpha)-total/(pr[j][SA1-1]*pow(beta,SA1-1));
			}

	
	double V = 0;
	for(int j=0;j<SP1;j++)
		for(int i=max(j-1,0);i<SA1;i++)
		{	V = V + pr[j][i]*pow(beta,i)*(T[j]*pow(K[j][i],alpha)- X[j][i]);
//			cout << j<<i<<" " << (T[j]*pow(K[j][i],alpha)- X[j][i]) << endl;
		}
	//qstat   << "V = "<< V << endl;
//	cout << "V = "<< V << endl;

	//cout << "	PI" <<  endl;
	double PI = 0;
	double qq;
	for(int j=0;j<SP1;j++)
		for(int i=max(j-1,0);i<SA1;i++)
		{	qq = 0;
			if(j==i || j==i)
				qq = Q[j];
			if(j-1==i)
				qq = Q[j-1];
			PI = PI + pr[j][i]*pow(beta,i)*(X[j][i] - w_tilde*K[j][i] - Cf(K[j][i],P[j][i]) - qq);
			//cout << j<<i<<": " <<(pr[j][i]*pow(beta,i)*(X[j][i] - w_tilde*K[j][i] - qq - Cf(K[j][i],P[j][i]) ))<< endl;
		}

//	cout <<"PI = " << PI << endl;


	double ICC[N1];
	//double LHS[N1];
	//double RHS[N1];
	double minICCc = MM;

	for(int n=0;n<SP1-1;n++)
	{
		ICC[n]= 0;
		
		for(int i=n;i<SA1;i++)
		{
			double prob=1.0;
			for(int it=n;it<i+1;it++)
				prob = prob*(1.0-P[n][it]);
					
			for(int j=n+1;j<min(SP1,i+2);j++)
			{	ICC[n] = ICC[n] + (pr[j][i]*pow(beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i])));
				//cout <<"ICC"<<n<<": " << j << ","<<i<<" = " << (pow(pr[j][i]*beta,i-n)*((T[j]*pow(K[j][i],alpha)- X[j][i])-(prob)*(T[j]*pow(K[n][i],alpha)- X[n][i]))) << endl; 		
			}	
		}	
		
		if(ICC[n]<minICCc)
			minICCc = ICC[n];

//		cout <<"ICC"<< n <<": " << ICC[n] << endl;
	}
						
	double LL[SP1-1];
	double minLLc = MM;
	for(int n=0;n<SP1-1;n++)
	{	LL[n]= T[n+1]*pow(K[n+1][n],alpha)-X[n+1][n];
		//qstat   << "LLC"<< n << " = " << LL[n] << endl;
	//	cout << "LLC"<< n << " = " << LL[n] << endl;
		if(LL[n]<minLLc)
	 		minLLc = LL[n];
	}
	///*
	double adjust = 0;
	//adjust = AD*pow(K[1][0]-K[2][1],2);

	//adjust = adjust + AD*pow(K[3][2]-K[2][1],2);
	//for(int n=0;n<SP1-1;n++)
	//	adjust = adjust + AD*pow(K[n+1][n]-K[n][n],2);

	double adjust2 = 0;
	//adjust2 = AD*pow(K[3][2]-K[4][3],2);

	//for(int j=0;j<SP1;j++)
	//	for(int n=j;n<SA1-1;n++)
	//		adjust2 = adjust2+ AD*pow(K[j][n]-K[j][n+1],2);
	//*/
	//double nada;
	//cin >> nada;
	//double nada1;
	//cin >> nada1;
	switch (iact) {
	case 0:
	*result = -V;
	break;
	case 1:
	*result = PI;
	break;
	case 2:
	*result = ubound- K[1][0];
	break;
	case 3:
	*result = ubound- K[2][1];
	break;
	case 4:
	*result = ubound- K[3][2];
	break;
	case 5:
	*result = ubound- K[4][3];
	break;
	case 6:
	*result = ubound- K[5][4];
	break;
	case 7:
	*result = ubound- K[6][5];
	break;
	case 8:
	*result = ubound- K[7][6];
	break;
	case 9:
	*result = ubound- K[8][7];
	break;
	case 10:
	*result = -lbound + K[1][0];
	break;
	case 11:
	*result = -lbound + K[2][1];
	break;
	case 12:
	*result = - lbound + K[3][2];
	break;
	case 13:
	*result = - lbound + K[4][3];
	break;
	case 14:
	*result = - lbound + K[5][4];
	break;
	case 15:
	*result = - lbound + K[6][5];
	break;
	case 16:
	*result = - lbound + K[7][6];
	break;
	case 17:
	*result = - lbound + K[8][7];
	break;
	case 18:
	*result =  ICC[0];
	break;
	case 19:
	*result =  ICC[1];
	break;
	case 20:
	*result =  ICC[2];
	break;
	case 21:
	*result =  ICC[3];
	break;
	case 22:
	*result =  ICC[4];
	break;
	case 23:
	*result =  ICC[5];
	break;
	case 24:
	*result =  ICC[6];
	break;
	case 25:
	*result =  ICC[7];
	break;
	case 26:
	*result =  LL[0];
	break;
	case 27:
	*result =  LL[1];
	break;
	case 28:
	*result =  LL[2];
	break;
	case 29:
	*result =  LL[3];
	break;
	case 30:
	*result =  LL[4];
	break;
	case 31:
	*result =  LL[5];
	break;
	case 32:
	*result =  LL[6];
	break;
	case 33:
	*result =  LL[7];
	break;

	default: ;
	break;
}
*ierr = 0;

return;
}



double Cf(double kx, double px)
{
	double xv;

	xv = w_tilde*pow(kx/z,gamma)*(1.0/(1.0-px)-1.0)*px; 
//	qstat   << kx << " " << px << " " << xv << endl;

	return xv;
}


double Cp(double kx, double px)
{
	double xv;

	xv = w_tilde*pow(kx/z,gamma)*((1.0/((1.0-px)*(1.0-px)))*px+ (1.0/(1.0-px)-1.0)); 

	return xv;
}


double Lfm(double kx, double px)
{
	double xv;

	xv = pow(kx,gamma)*(1.0/(1.0-px)-1.0)*px/z; 

	return xv;
}


double maxi(double x, double y)
{

	double V = x;
	if(y>x)
		V = y;
	return V;
